(*
 *        descript.ion
 *     
 *)
program mp3ToIon;
{$APPTYPE CONSOLE}
uses
  SysUtils,
  classes,
  Common in 'Common.pas',
  ID3v1 in 'ID3v1\ID3v1.pas',
  ID3v2 in 'ID3v2\ID3v2.pas',
  OggVorbis in 'OggVorbis\OggVorbis.pas',
  WMAfile in 'WMAfile\WMAfile.pas';

Const
 DefaultDescriptionPattern='%2 -- %1';
 DescriptionPatternFileName='mp3ToIon.ini';
Type
 PDescriptionCage=^TDescriptionCage;
 TDescriptionCage=record
                   FileName:string;
                   Description:string;
                   Quoted:boolean;
                  end;{TDescriptionCage}
 (*
  *     
  *)
 TTagProcessor=function (FileName:String):String;
Var
 DescriptionFileName:string;
 DescriptionList:TList;
 DescriptionPattern:string;

 (*
  * Credits
  *)
 procedure PrintAbout;
 begin
  Writeln('Program is intended to create/update *.mp3-files description');
  Writeln('Copyright (c) 2004 Bushuev S.A.');
  Writeln('Was used "Audio Tools Library" Copyright (c) 2001,2002 by Jurgen Faul');
  Writeln('To change description pattern create file mp3ToIon.ini with one line');
  Write('Where %1 will be substituted by Title, ');
  Write('%2 -- Artist, ');
  Write('%3 -- Year, ');
  Write('%4 -- Album, ');
  Write('%5 -- Genre, ');
  Writeln('%6 -- Track');
 end;{PrintAbout}
 (******************************************************************************)

 (*
  *    ,     
  *)
 function GetDescriptionFileName: string;
  Var
   OutFile:File;
 begin
  if FileExists('Descript.ion')
   then
    begin
     GetDescriptionFileName:='Descript.ion';
    end {THEN}
   else
    begin
     if FileExists('files.bbs')
      then
       begin
        GetDescriptionFileName:='files.bbs';
       end {THEN}
      else
       begin
        GetDescriptionFileName:='Descript.ion';
        (*
         *    
         *)
        AssignFile(OutFile,'Descript.ion');
        Rewrite(OutFile);
        CloseFile(OutFile);
       end;{ELSE}
    end;{ELSE}
 end;{GetDescriptionFileName}
 (******************************************************************************)

 (*
  * ,       
  *)
 function ParseDescriptionLine(DescLine:string; DescCage:PDescriptionCage): boolean;
 Var
  Est:boolean;
  FirstSpace:integer;
  NextQuote:integer;
 begin
  Est:=True;
  DescCage^.Quoted:=False;

  if Length(DescLine)>2 // : x_x
   then
    begin
     (*
      *   
      *)
     if DescLine[1]='"' //   
      then
       begin
        DescCage^.Quoted:=True;
        Delete(DescLine,1,1);// 
        NextQuote:=Pos('"',DescLine);
        if NextQuote>0
         then
          begin
           DescCage^.FileName:=Copy(DescLine,1,NextQuote-1);
           Delete(DescLine,1,NextQuote);//    
          end {THEN}
         else
          begin
           Est:=False;
          end;{ELSE}
       end {THEN}
      else
       begin
        {Just Name, without Spaces}
        FirstSpace:=Pos(' ',DescLine);
        if FirstSpace>0 //    
         then
          begin
           DescCage^.FileName:=Copy(DescLine,1,FirstSpace-1);
           Delete(DescLine,1,FirstSpace);// 
           if Length(DescCage^.FileName)=0
            then
             begin
              Est:=False;
             end;{THEN}
          end {THEN}
         else
          begin
           Est:=False;
          end;{ELSE}
       end;{ELSE}

     {Handle Description}
     if Est
      then
       begin
        // 
        DescLine:=Trim(DescLine);
        if Length(DescLine)>0
         then
          begin
           DescCage^.Description:=DescLine;
          end {THEN}
         else
          begin
           Est:=False;
          end;{ELSE}
       end;{THEN}
    end {THEN}
   else
    begin
     Est:=False;
    end;{ELSE}
  ParseDescriptionLine:=Est;
 end;{ParseDescriptionLine}
 (******************************************************************************)
 (*
  *   
  *)
 function GetDescriptionPattern:string;
 Var
  Est:string;
  InFile:TextFile;
 begin
  Est:=DefaultDescriptionPattern;
  if FileExists(DescriptionPatternFileName)
   then
    begin
     AssignFile(InFile,DescriptionPatternFileName);
     Reset(InFile);
     Readln(InFile,Est);
     CloseFile(InFile);
    end;{THEN}
  GetDescriptionPattern:=Est;
 end;{GetDescriptionPattern}
 (******************************************************************************)
 (*
  *      
  *)
 procedure Init;
 Var
  InFile:TextFile;
  DescriptionLine:string;
  DescriptionCage:PDescriptionCage;
 begin
  PrintAbout;

  (*
   *   
   *)
  DescriptionPattern:=GetDescriptionPattern;


  DescriptionFileName:=GetDescriptionFileName;
  DescriptionList:=TList.Create;
  DescriptionList.Capacity:=121;
  AssignFile(InFile, DescriptionFileName);
  Reset(InFile);
  while Not(EOF(InFile)) do
   begin
    ReadLn(InFile,DescriptionLine);
    New(DescriptionCage);
    if ParseDescriptionLine(DescriptionLine, DescriptionCage)
     then
      begin
       DescriptionList.Add(DescriptionCage);
      end {THEN}
     else
      begin
       Dispose(DescriptionCage);
      end;{ELSE}
   end;{WHILE}
  CloseFile(InFile);
 end;{Init}
 (******************************************************************************)

 (*
  *     
  * -1   
  *)
 function isFileNameInList(FileName:string): integer;
 Var
  f:integer;
  Position:integer;
 begin
  Position:=-1;
  for f:=0 to DescriptionList.Count-1 do
   begin
    if (PDescriptionCage(DescriptionList.Items[f])^.FileName=(FileName))
     then Position:=f;
   end;{FOR}
  isFileNameInList:=Position;
 end;{isFileNameInList}
 (******************************************************************************)

 (*
  *      
  *)
 function ComposeDescription(Title, Artist,Year,Album,Genre:string; Track:integer): string;
 {                            %1      %2    %3    %4   %5             %6}
 Var
  Est:string;
 begin
{  ComposeDescription:=Artist+' -- '+Title;}
  Est:=DescriptionPattern;
  Est:=StringReplace(Est,'%1',Title,[rfReplaceAll]);
  Est:=StringReplace(Est,'%2',Artist,[rfReplaceAll]);
  Est:=StringReplace(Est,'%3',Year,[rfReplaceAll]);
  Est:=StringReplace(Est,'%4',Album,[rfReplaceAll]);
  Est:=StringReplace(Est,'%5',Genre,[rfReplaceAll]);
  Est:=StringReplace(Est,'%6',IntToStr(Track),[rfReplaceAll]);
  ComposeDescription:=Est;
 end;{ComposeDescription}
 (******************************************************************************)
 (*
  *   MP3    
  *)
 function ProcessMP3Tags(FileName:string): string;
 Var
  ID3v2:TID3v2;
  ID3v1:TID3v1;
  DescLine:string;
 begin
  DescLine:='';
  ID3v1:=TID3v1.Create;
  ID3v2:=TID3v2.Create;
  ID3v1.ReadFromFile(FileName);
  ID3v2.ReadFromFile(FileName);

  (*
   *    ,      
   *)
  if ID3v2.Exists
   then
    begin
     DescLine:=ComposeDescription(ID3v2.Title,ID3v2.Artist,ID3v2.Year,ID3v2.Album,ID3v2.Genre,ID3v2.Track);
    end {THEN}
   else
    begin
     if ID3v1.Exists
      then
       begin
        DescLine:=ComposeDescription(ID3v1.Title,ID3v1.Artist,ID3v1.Year,ID3v1.Album,ID3v1.Genre,ID3v1.Track);
       end;{THEN}

    end;{ELSE}


  ID3v1.Free;
  ID3v2.Free;
  ProcessMP3Tags:=DescLine;
 end;{ProcessMP3Tags}
 (******************************************************************************)
 (*
  *   Ogg    
  *)
 function ProcessOggTags(FileName:string): string;
 Var
  ID:TOggVorbis;
  DescLine:string;
 begin
  DescLine:='';
  ID:=TOggVorbis.Create;
  ID.ReadFromFile(FileName);

  (*
   *    ,      
   *)
  if ID.Valid
   then
    begin
     DescLine:=ComposeDescription(ID.Title,ID.Artist,ID.Date,ID.Album,ID.Genre,ID.Track);
    end; {THEN}
  ID.Free;
  ProcessOggTags:=DescLine;
 end;{ProcessOggTags}
 (******************************************************************************)

 (*
  *   WMA    
  *)
 function ProcessWMATags(FileName:string): string;
 Var
  ID:TWMAfile;
  DescLine:string;
 begin
  DescLine:='';
  ID:=TWMAfile.Create;
  ID.ReadFromFile(FileName);

  (*
   *   
   *)
  if ID.Valid
   then
    begin
     DescLine:=ComposeDescription(ID.Title,ID.Artist,ID.Year,ID.Album,ID.Genre,ID.Track);
    end; {THEN}
  ID.Free;
  ProcessWMATags:=DescLine;
 end;{ProcessWMATags}
 (******************************************************************************)

 (*
  *        
  *)
 procedure ProcessTagForTheGivenType(TypeExtention:string; TagProcessor:TTagProcessor);
 Var
  SearchRec:TSearchRec;
  Est,f:integer;
  DescCage:PDescriptionCage;
 begin
  Est:=FindFirst(TypeExtention,faAnyFile{-faDirectory},SearchRec);
  while Est=0 do
   begin
    f:=isFileNameInList(Win2DosString(SearchRec.Name));
    if f>=0
     then
      begin
       {FileName in list}
       DescCage:=PDescriptionCage(DescriptionList.Items[f]);
      end {THEN}
     else
      begin
       New(DescCage);
       DescCage^.FileName:=Win2DosString(SearchRec.Name);
       DescriptionList.Add(DescCage);
       if Pos(' ',SearchRec.Name)>0
        then DescCage^.Quoted:=True
        else DescCage^.Quoted:=False;
      end;{ELSE}

    DescCage^.Description:=Win2DosString(TagProcessor(SearchRec.Name));
    if Length(DescCage^.Description)=0
     then
      begin
       f:=DescriptionList.IndexOf(DescCage);
       if f>=0
        then
         begin
          DescriptionList.Delete(f);
          Dispose(DescCage);
         end {THEN}
        else
         begin
          Writeln('Description not found in list!!!!!');
         end;{ELSE}

      end;{THEN}
    Est:=FindNext(SearchRec);
   end;{WHILE}

  FindClose(SearchRec);
 end;{ProcessTagForTheGivenType}
 (******************************************************************************)

 (*
  *     
  *)
 procedure Work;
 begin
  (*
   *   MP3 
   *)
  ProcessTagForTheGivenType('*.mp3',ProcessMP3Tags);
  (*
   *   MP2 
   *)
  ProcessTagForTheGivenType('*.mp2',ProcessMP3Tags);
  (*
   *   Ogg 
   *)
  ProcessTagForTheGivenType('*.ogg',ProcessOggTags);
  (*
   *   WMA 
   *)
  ProcessTagForTheGivenType('*.wma',ProcessWMATags);
 end;{Work}
 (******************************************************************************)

 (*
  *   
  *)
 function GenerateDescriptionFileLine(DescCage:PDescriptionCage): String;
 begin
  if DescCage^.Quoted
   then
    GenerateDescriptionFileLine:='"'+(DescCage^.FileName)+'" '+DescCage^.Description
   else
    GenerateDescriptionFileLine:=(DescCage^.FileName)+' '+DescCage^.Description;
 end;{GenerateDescriptionFileLine}
 (******************************************************************************)
 (*
  *   
  *)
 procedure Done;
 Var
  f:integer;
  OutFile:TextFile;
 begin
  AssignFile(OutFile,DescriptionFileName);
  ReWrite(OutFile);
  for f:=1 to DescriptionList.Count do
   begin
    Writeln(OutFile, GenerateDescriptionFileLine(DescriptionList.Items[f-1]));
    Dispose(DescriptionList.Items[f-1]);
   end;{FOR}
  CloseFile(OutFile);
  DescriptionList.Free;
 end;{Done}
 (******************************************************************************)

begin
  Init;
  Work;
  Done;
end.
